@@ -26,7 +26,7 @@ summary.ldblm <-function(object,...){
     R2 <- 1 - sum(weights*(z$fitted.values-z$y)^2)/(sum(weights*(y0^2)))
 
     # trace_hat
-    t_hat <- sum(diag(z$Shat))
+    t_hat <- sum(diag(z$S))
 
     # residual sigma
     sigma<-attr(z,"sigma")
@@ -39,10 +39,34 @@ summary.ldblm <-function(object,...){
      family <-object$family
     else 
      family <-"gaussian"
+     
+     
+    kind.kernel <- switch(attr(object,"kind.of.kernel"),
+     "(1) Epanechnikov",
+	   "(2) Biweight",
+		 "(3) Triweight",
+		 "(4) Normal",
+		 "(5) Triangular",
+		 "(6) Uniform")
+    
+      crit.value <- NULL
+      if (!is.null(attr(object,"OCV_opt")))
+        crit.value <- attr(object,"OCV_opt") 
+      if (!is.null(attr(object,"GCV_opt")))
+        crit.value <- attr(object,"GCV_opt") 
+      if (!is.null(attr(object,"AIC_opt")))
+        crit.value <- attr(object,"AIC_opt") 
+      if (!is.null(attr(object,"BIC_opt")))
+        crit.value <- attr(object,"BIC_opt")
+         
+      
+     
     
     # list to be returned
     ans <- list(nobs=nobs,r.squared=R2,trace.hat=t_hat,call=call,
-                  residuals=z$residuals,family=family,y=z$y)
+                  residuals=z$residuals,family=family,y=z$y,
+                  kind.kernel=kind.kernel,method=attr(object,"method"),
+                  h_opt=object$h_opt, crit.value=crit.value)
     class(ans)<-"summary.ldblm"
 
     return(ans)
